;;; dinbrief.el - Special code for LaTeX-Style dinbrief.

;; Contributed by Werner Fink <tex@itap.physik.uni-stuttgart.de>
;; Please direct comments to him.
;;
;; Change to new dinbrief.sty -- Werner Fink <werner@suse.de>
;;                               ... yes, we are identical

;;; Commentary:

;; LaTeX-Style: dinbrief.sty
;;      Server: rusinfo.rus.uni-stuttgart.de
;;   Directory: /pub/soft/tex/macros/latex/contrib/letters

;;; Code:

(TeX-add-style-hook "dinbrief"
 (function
  (lambda ()
    (add-hook 'LaTeX-document-style-hook
     'LaTeX-style-dinbrief)
    (LaTeX-add-environments
     '("letter" LaTeX-recipient-hook)
     "dinquote")
    (TeX-add-symbols
     '("address" "Absender: ")
     '("postremark" "Postvermerk: ")
     '("date" "Datum: ")
     '("subject" "Betreff: ")
     '("handling" "Behandlungsvermerk: ")
     '("cc" "Verteiler: ")
     '("place" "Heutiger Ort: ")
     "makelabels"
     "nowindowrules"
     "windowrules"
     "nowindowtics"
     "windowtics"
     "disabledraftstandard"
     "enabledraftstandard"
     "centeraddress"
     "normaladdress"
     '("encl" "Anlagen: ")
     '("backaddress" "Retouradresse: ")
     '("signature" "Unterschrift: ")
     '("opening" "Anrede: ")
     '("closing" "Schluss: ")))))

(if (string-match "XEmacs" emacs-version)
    (defun TeX-deactivate-mark () (zmacs-deactivate-region))
  (defun TeX-deactivate-mark () (deactivate-mark)))

(defmacro din-insert (&rest args)
  "Insert text ignoring active markers."
  `(progn (if (TeX-mark-active) (TeX-deactivate-mark))
     (insert ,@args)))

(defun LaTeX-style-dinbrief ()
  "Insert some useful packages for writing german letters."
  (if (equal LaTeX-version "2e")
    (save-excursion
      (goto-char (point-min)) ; insert before \begin{document}
      (if (re-search-forward ".begin.document." (point-max) t)
          (beginning-of-line 1))
      (open-line 2)
      (indent-relative-maybe)
	(din-insert TeX-esc "usepackage"
		LaTeX-optop "latin1" LaTeX-optcl
		TeX-grop "inputenc" TeX-grcl)
	(newline-and-indent)
	(din-insert TeX-esc "usepackage"
		LaTeX-optop "T1" LaTeX-optcl
		TeX-grop "fontenc" TeX-grcl)
	(newline-and-indent)
	(din-insert TeX-esc "usepackage"
		TeX-grop "ngerman" TeX-grcl)
	(newline-and-indent)
	(din-insert TeX-esc "usepackage"
		TeX-grop "umlaut" TeX-grcl)
	(TeX-run-style-hooks "inputenc")
	(TeX-run-style-hooks "fontenc")
	(TeX-run-style-hooks "ngerman")
	(TeX-run-style-hooks "umlaut"))))

(defun LaTeX-recipient-hook (environment)
  "Insert ENVIRONMENT and prompt for recipient and address."
  (let (
	(sender (LaTeX-sender))
	(recipient (read-string "Empfänger: "))
	(address (LaTeX-recipient))
	(date (read-string "Datum: " (LaTeX-today)))
	(postremark (read-string "Postvermerk: "))
	(fenster (read-string "Fenster \(ja/nein\): "))
	(vermerk (read-string "Behandlungsvermerk: "))
	(verteil (read-string "Verteiler: "))
	(betreff (read-string "Betreff: "))
	(opening (read-string "Anrede: "))
	(closing (read-string "Schluss: "))
	(signature (read-string "Unterschrift: "))
	(anlage (read-string "Anlagen: "))
	)

    (if (string= fenster "ja")
	(progn
	  (din-insert TeX-esc "enabledraftstandard")
	  (newline-and-indent)
	  (din-insert TeX-esc "centeraddress")
	  (newline-and-indent)
	  (din-insert TeX-esc "nowindowrules")
	  (newline-and-indent)
	  (din-insert TeX-esc "windowtics")
	  (newline-and-indent)
	  (let ((retouradr (read-string "Retouradresse: " sender)))
	    (newline-and-indent)
	  (if (not (zerop (length retouradr)))
	      (progn
		(if (TeX-mark-active) (TeX-deactivate-mark))
		(din-insert TeX-esc "backaddress" TeX-grop retouradr TeX-grcl)
		(newline-and-indent)))))
      (din-insert TeX-esc "enabledraftstandard")
      (newline-and-indent)
      (din-insert TeX-esc "centeraddress")
      (newline-and-indent)
      (din-insert TeX-esc "nowindowrules")
      (newline-and-indent)
      (din-insert TeX-esc "windowtics"))
      (newline-and-indent)
    (if (not (zerop (length signature)))
	(progn
	  (din-insert TeX-esc "signature" TeX-grop signature TeX-grcl)
	  (newline-and-indent)))
    (if (not (zerop (length date)))
	(progn
	  (din-insert TeX-esc "date" TeX-grop date TeX-grcl)
	  (newline-and-indent)))
    (newline-and-indent)

    (let ((indentation (current-column)))
      (LaTeX-insert-environment
       environment
       (concat TeX-grop recipient
	       (if (not (zerop (length address)))
		   (concat
		    (if (not (zerop (length recipient)))
			(concat " " TeX-esc TeX-esc " "))
		    address))
	       TeX-grcl))
      (save-excursion			; Fix indentation of address
	(if (search-backward TeX-grcl nil 'move)
	    (let ((addr-end (point-marker)))
	      (if (search-backward TeX-grop nil 'move)
		  (let ((addr-column (current-column)))
		    (while (search-forward
			    (concat TeX-esc TeX-esc)
			    (marker-position addr-end) 'move)
		      (progn
			(newline)
			(indent-to addr-column))))))))
      (din-insert "\n")
      (indent-to indentation))
    (if (not (zerop (length postremark)))
	(progn
	  (din-insert TeX-esc "postremark" TeX-grop postremark TeX-grcl)
	  (newline-and-indent)))
    (if (not (zerop (length betreff)))
	(progn
	  (din-insert TeX-esc "subject" TeX-grop)
	    (if (equal LaTeX-version "2e")
	      (din-insert TeX-esc "textbf" TeX-grop betreff TeX-grcl TeX-grcl)
	      (din-insert betreff TeX-grcl))
	  (newline-and-indent)))
    (if (not (zerop (length vermerk)))
	(progn
	  (din-insert TeX-esc "handling" TeX-grop vermerk TeX-grcl)
	  (newline-and-indent)))
    (if (not (zerop (length verteil)))
	(progn
	  (din-insert TeX-esc "cc" TeX-grop verteil TeX-grcl)
	  (newline-and-indent)))
    (if (not (zerop (length anlage)))
	(progn
	  (din-insert TeX-esc "encl" TeX-grop anlage TeX-grcl)
	  (newline-and-indent)))
    (din-insert TeX-esc "opening"
	    TeX-grop
	    (if (zerop (length opening))
		(concat TeX-esc " ")
	      opening)
	    TeX-grcl "\n")

    (indent-relative-maybe)
    (save-excursion
      (din-insert "\n" TeX-esc "closing"
	      TeX-grop
	      (if (zerop (length closing))
		  (concat TeX-esc " ")
		closing)
	      TeX-grcl "\n")
      (indent-relative-maybe))))

(defun LaTeX-sender nil
  "Read and writes the senders address"
  (interactive)
  (let ((name (read-string "Absender: " (user-full-name)))
	(str  (read-string "Meine Strasse:  "))
	(ort  (read-string "Mein Wohnort:  ")))
    (if (not (zerop (length name)))
	(progn
	  (goto-char (point-min)) ; insert before \end{document}
	  (if (re-search-forward ".end.document." (point-max) t)
	     (beginning-of-line 1))
	  (previous-line 1)
	  (din-insert TeX-esc "address" TeX-grop name)
	  (if (not (zerop (length str)))
	      (progn
		(din-insert " " TeX-esc TeX-esc)
		(newline-and-indent)
		(din-insert str)))
	  (if (not (zerop (length ort)))
	      (progn
		(din-insert " " TeX-esc "par")
		(newline-and-indent)
		(din-insert ort)))
	  (din-insert TeX-grcl)
	  (newline-and-indent)
	  (concat name ", " str ", " ort)))))

(defun LaTeX-recipient nil
  "Read and returns the recipient address"
  (interactive)
  (let ((str  (read-string "Wohnhaft in Strasse:  "))
	(ort  (read-string "Aus der Ortschaft:  ")))
    (if (not (zerop (length str)))
	(if (not (zerop (length ort)))
	    (concat str " " TeX-esc TeX-esc " " ort)
	  str)
      (if (not (zerop (length ort)))
	  ort))
    )
)

(defun LaTeX-today nil
  "Return a string representing todays date according to flavor."
  (interactive)
   (let ((ctime-string (current-time-string))
	(month-alist '(("Jan" . "Januar")
		       ("Feb" . "Februar")
		       ("Mar" . "M\\\"arz")
		       ("Apr" . "April")
		       ("May" . "Mai")
		       ("Jun" . "Juni")
		       ("Jul" . "Juli")
		       ("Aug" . "August")
		       ("Sep" . "September")
		       ("Oct" . "Oktober")
		       ("Nov" . "November")
		       ("Dec" . "Dezember"))))
    (string-match
     "^\\S-+\\s-+\\(\\S-+\\)\\s-+\\(\\S-+\\)\\s-+\\S-+\\s-+\\(\\S-+\\)"
     ctime-string)
    (let ((year (substring ctime-string (match-beginning 3) (match-end 3)))
	  (month (substring ctime-string (match-beginning 1) (match-end 1)))
	  (day (substring ctime-string (match-beginning 2) (match-end 2)))
	  (place (read-string "Heutiger Ort: ")))
      (if (assoc month month-alist)
	  (progn
	    (setq month (cdr (assoc month month-alist)))
	    (if (> 2 (length day))
		(setq day (concat "0" day)))))
      (format "%s, den %s. %s %s" place day month year))))

;;; dinbrief.el ends here
